import { useState, useMemo, useRef, useEffect } from 'react';
import { useFrame } from '@react-three/fiber';
import { useFBX, useAnimations } from '@react-three/drei';
import * as THREE from 'three';
import {
    MODEL_BOX_SCALE,
    SHELF_BAY_GROUP_COUNT,
    SHELF_BAY_UNIT_SEPARTE_DISTANCE,
    SHELF_LEV_UNIT_SEPARTE_DISTANCE,
    SHELF_HEIGHT_FROM_GROUND
} from '@/config/setting'
import * as Common from '../utils/common';
import { useStore } from '@/store';

const getShelfRow = (row) => {
    return row;
}

const getShelfBay = (bay) => {
    return bay - (bay - 1) % SHELF_BAY_GROUP_COUNT;
}

const getShelfLev = (lev) => {
    return 1;
}

const getShelfNo = (row, bay, lev) => {
    return Common.generateLocNo(getShelfRow(row), getShelfBay(bay), getShelfLev(lev));
}

const Box = (props) => {
    const { row, bay, lev, position = [0, 300, 0], rotationY = 0 } = props;
    const state = useStore();

    const [pos, setPos] = useState([]);
    const [rotaY, setRotaY] = useState(0);

    const boxModel = useMemo(() => {
        const fbx = useFBX('/models/box/box.fbx');
        if (!fbx.castShadow) {
            Common.setShadow(fbx);
        }
        fbx.scale.set(MODEL_BOX_SCALE, MODEL_BOX_SCALE, MODEL_BOX_SCALE);
        return fbx.clone();
    }, [])

    useEffect(() => {
        if (row && bay && lev) {
            // from shelf
            const shelfNo = getShelfNo(row, bay, lev);
            const shelfPos = state.shelfList[shelfNo];
            if (!shelfPos) { return }
            const { row: shelfRow, bay: shelfBay, lev: shelfLev } = Common.parseLocNo(shelfNo);

            // cal bay position
            let posZByLev = shelfPos[2];
            if (bay === shelfBay) {
                posZByLev = shelfPos[2] + SHELF_BAY_UNIT_SEPARTE_DISTANCE;
            }
            if (bay === shelfBay + 1) {
                posZByLev = shelfPos[2];
            }
            if (bay === shelfBay + 2) {
                posZByLev = shelfPos[2] - SHELF_BAY_UNIT_SEPARTE_DISTANCE;
            }

            setPos([
                shelfPos[0],
                shelfPos[1] + SHELF_HEIGHT_FROM_GROUND + (lev - shelfLev) * SHELF_LEV_UNIT_SEPARTE_DISTANCE,
                posZByLev
            ]);
            setRotaY(Math.PI / 2);
        } else {
            // from agv
            setPos(position);
            setRotaY(rotationY);
        }
    }, [props]);

    return (
        <>
            <group rotation-y={rotaY} position={pos}>
                <primitive object={boxModel} castShadow />
            </group>
        </>
    )
}

export default Box;